from collections import OrderedDict


NOTEBOOK_JOB_CONFIGURATION = {'module': 'main', 'package_path': None, 'package_manager': 'pip',
                              'requirements': 'requirements.txt', 'time_limit': 2880, 'datasets_set': [], 'code': None,
                              'code_commit': 'some_hash', 'module_arguments': {}, 'framework': {'slug': 'jupyter'},
                              'python_version': '2.7', 'mode': 'single', 'workers': {'slug': 't2.small', 'replicas': 1},
                              'name': 'snowy-surf-115', 'description': ''}

NOTEBOOK_API_RESPONSE = OrderedDict([
    ('job_id', '853b9f10-36ce-4de4-b2f8-108d69733b42'),
    ('repository', None),
    ('repository_owner', None),
    ('repository_name', None),
    ('repository_owner_photo_url', None),
    ('display_name', 'winter-leaf-119'),
    ('description', ''),
    ('created_at', '2018-05-08T11:32:26.354821Z'),
    ('launched_at', None),
    ('terminated_at', None),
    ('modified_at', '2018-05-08T11:32:26.365201Z'),
    ('current_time_left_percentage', 0),
    ('datasets_set', []),
    ('members', [OrderedDict([
        ('id', 12044),
        ('access', 50),
        ('access_level', 'owner'),
        ('status', 'AC'),
        ('user', 795),
        ('username', 'allgreed'),
        ('photo_url',
         'https://tensorport-api-development.s3.amazonaws.com/images/empty_image_400_400.png'
         ),
        ('email', 'whatever@example.com'),
        ('first_name', 'John'),
        ('last_name', 'Kowalsky'),
    ])]),
    ('total_running_time', 0),
    ('unpaid_running_time', 0),
    ('git_commit_hash', 'some_hash'),
    ('git_commit', None),
    ('created_by', 795),
    ('logs', None),
    ('current_run', None),
    ('runs', []),
    ('tags', []),
    ('user_panel', None),
    ('status', 'created'),
    ('parameters', OrderedDict([
        ('framework', OrderedDict([('slug', 'jupyter'), ('name',
                                                         'jupyter'), ('version', '1.0.0')])),
        ('python_version', '2.7'),
        ('package_manager', 'pip'),
        ('mode', 'notebook'),
        ('requirements', 'requirements.txt'),
        ('time_limit', 2880),
        ('package_path', ''),
        ('module_arguments', OrderedDict()),
        ('module', 'main'),
        ('datasets', []),
        ('workers', OrderedDict([('slug', 't2.small'), ('replicas', 1),
                                 ('queue', 'tensorport-jobmaster-blessed'), ('type', 't2.small'
                                                                             ), ('blessed', True)])),
        ('token', '85c88d21-fb18-42be-8a1a-f73da4585a02'),
    ])),
    ('resources', None),
    ('notebook_url',
     'http://853b9f10-36ce-4de4-b2f8-108d69733b42.jupyter.v2.clusterone.com'
     ),
])



from clusterone.persistance.session import Session

import functools


def authenticate():
    def decorator(f):

        @functools.wraps(f)
        def wrapper(*args, **kwargs):
            try:
                session = args[0].config
            except:
                session = Session()
                session.__init__()
                session.load()

            username = session.get('username')
            token = session.get('token')
            if username == '' or token == '':
                print("You are not logged in yet. Use 'just login'.")
                return
            return f(*args, **kwargs)

        return wrapper

    return decorator




import py
import click
import json

import os
HOME_DIR = os.getcwd()

#TODO: Redo this to sesion / sth else
#TODO: Maybe as a context manager
class Session(dict):
    def __init__(self, *args, **kwargs):
        self.config = py.path.local(
            click.get_app_dir('clusterone')).join('session.json')

        self.home = HOME_DIR

        # retry behaviour, used when aquiring tport remote repo
        self.retry_count = 60 # times
        self.retry_interval = 1 # seconds

        # events watch behaviour, used when dispalying events
        self.events_refresh_rate = 1 # seconds

        super(Session, self).__init__(*args, **kwargs)


    def load(self):
        """load the JSON session file from disk"""
        # TODO: Context manager?
        try:
            self.update(json.loads(self.config.read()))
            # logger.info('Session file loaded.')

        # When failing to parse the file we'd like this not to fail
        # User will login then and rewrite the session
        except (ValueError, py.error.ENOENT) as exception:
            pass

    def save(self):
        #TODO: Is this necessary?
        self.config.ensure()
        with self.config.open('w', encoding='utf-8') as dumpfile:
            try:
                dumpfile.write(json.dumps(self))
            except TypeError:
                # Python 2 compliance - .write() required unicode
                dumpfile.write(json.dumps(self).decode('utf-8'))

    def delete(self):
        try:
            self.config.remove()
        except py.error.ENOENT:
            # no file -> deletion done
            pass




from .client_exceptions import JobNameConflict

def test_name_conflict_error_message(mocker):
    exception = JobNameConflict(['6789-6789-6789', '4567-4567-4567'])

    assert str(exception) == """Job name resolves to multiple IDs:
6789-6789-6789
4567-4567-4567
Cannot proceed. Please rerun this command with one of the IDs above."""




import click

from clusterone import authenticate
from clusterone.utilities import path_to_job_id, make_table, serialize_job


HEADER = ['Property', 'Value']

HEADER_BASE = [
    'Name',
    'Status',
    'Project',
    'Module',
    #'Package-path',
    #'Datasets',
    'Python version',
    'Package manager',
    'Requirements',
    'Framework',
    'Framework version',
    'Mode',
    'Worker type',
    'Worker replicas',
    'Ps type',
    'Ps replicas',
    'Time limit',
    ]

#TODO: Test this thoughrilly in the future -> [], [dataset], [dataset] * 3
def displayable_data(datasets_list):
    return "".join(["{}:{}\n".format(dataset['mount_point'], dataset['hash']) for dataset in datasets_list])

def extract_data_from_job(job):

    mode = job['parameters']['mode']

    excluded_fields = \
        ["Ps type", "Worker replicas", "Ps replicas"] if mode == "single" \
        else []

    extracted_data = [
        job['display_name'],
        job['status'],
        job['repository_name'],
        job['parameters']['module'],
        #job['parameters']['package_path'],
        #displayable_data(job['parameters']['datasets']),
        job['parameters']['python_version'],
        job['parameters']['package_manager'],
        job['parameters']['requirements'],
        job['parameters']['framework']['name'],
        job['parameters']['framework']['version'],
        mode,
        job['parameters']['workers']['type'],
        job['parameters']['workers']['replicas'],
        job['parameters'].get('parameter_servers', {}).get('type'),
        job['parameters'].get('parameter_servers', {}).get('replicas'),
        "{} minutes".format(job['parameters']['time_limit']),
    ]

    key_value_pairs = zip(HEADER_BASE, extracted_data)

    #TODO: Redo this to namedtuples -> this pair[0] looks like shit...
    return [pair for pair in key_value_pairs if pair[0] not in excluded_fields]

@click.command()
@click.pass_obj
@authenticate()
@click.argument(
    'job-path-or-id',
    )
def command(context, job_path_or_id):
    """
    Get information about a job
    """

    job = serialize_job(job_path_or_id, context=context)

    click.echo(make_table(extract_data_from_job(job), header=HEADER))

    return job



from click.testing import CliRunner
from clusterone import ClusteroneClient

from clusterone.commands.create.project import cmd
from clusterone.clusterone_cli import cli

def test_all_parameters(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    cmd.main = mocker.Mock()

    CliRunner().invoke(cli, ['create', 'project', 'someProjectName', '--description', 'This is a sample project description'])

    cmd.main.assert_called_with(mocker.ANY, 'someProjectName', 'This is a sample project description')

def test_name_only(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    cmd.main = mocker.Mock()

    CliRunner().invoke(cli, ['create', 'project', 'test'])

    cmd.main.assert_called_with(mocker.ANY, 'test', '')

def test_message(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    cmd.main = mocker.Mock()

    result = CliRunner().invoke(cli, ['create', 'project', 'whatever-really'])

    assert 'Project creating, this might take up to a minute' in result.output

def test_outputs_remote_url(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    cmd.main = mocker.Mock(return_value="sample.git.link.remote.git")

    result = CliRunner().invoke(cli, ['create', 'project', 'whatever-really'])

    assert "sample.git.link.remote.git" in result.output




# -*- coding: utf-8 -*-

import click
from click import launch
from clusterone import client


@click.command()
@click.pass_obj
def command(context):
    """
    Open the Matrix in the browser
    """

    #TODO: What if operating in headless mode? What does happens then?
    launch(client.matrix_url)



import click
from click import echo

from clusterone import authenticate
from clusterone.just_types import Notebook
from clusterone import client


@click.command()
@click.pass_obj
@authenticate()
@click.argument(
    'notebook',
    type=Notebook(),
)
def command(context, notebook):
    """
    Starts an existing notebook

    NOTEBOOK: path or uuid of an existing notebook
    """

    notebook.start(client)
    echo(notebook.id)



import click


def lazify(function):
    """
    A decorator transforming function returning an iterable into generator

    Extremely usefull is the list is constructed based on request
    """

    def generator(*args, **kwargs):
        for item in function(*args, **kwargs):
            yield item

    return generator


def lazy_property(function):
    """
    Convinience wrapper: combines @property and @lazify
    """

    def wrapper(*args, **kwargs):
        return function(*args, **kwargs)
    return property(lazify(wrapper))


class Choice(click.Choice):
    """
    Extends click.Choice so that a generator iterator can be passed
    The generator is evaluated only once and only when needed
    """

    def __init__(self, choices):
        """
        Adds indirection level to hook into values lookup
        The super() is not called deliberately
        """

        self._choices = choices

    @property
    def choices(self):
        # type: () -> [str]
        #TODO: Move this to function signature when dropping Python 2.7 compliance
        """
        Caching generator as a list
        """

        self._choices = list(self._choices)

        return self._choices



from pytest import raises
from click.exceptions import BadParameter
from coreapi.exceptions import NetworkError

from clusterone.persistance import config
from clusterone import ClusteroneClient


def test_normalize_config_keys(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    config.Config.__init__ = mocker.Mock(return_value=None)
    config.Config.set = mocker.Mock()
    test_config = config.Config()

    test_config.ENDPOINT = "http://elorap.com/api"

    test_config.set.assert_called_with('endpoint', "http://elorap.com/api/")


def test_endpoint_slash_and_api(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    config.Config.__init__ = mocker.Mock(return_value=None)
    config.Config.set = mocker.Mock()
    test_config = config.Config()

    test_config.endpoint = "http://elorap.com"

    test_config.set.assert_called_with('endpoint', "http://elorap.com/api/")


def test_endpoint_verify_schema(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    config.Config.__init__ = mocker.Mock(return_value=None)
    config.Config.set = mocker.Mock()
    test_config = config.Config()

    test_config.endpoint = "http://elorap.com"

    ClusteroneClient.__init__.assert_called_with(mocker.ANY, api_url="http://elorap.com/api/")


def test_endpoint_invalid_schema_throw(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None, side_effect=NetworkError())
    config.Config.__init__ = mocker.Mock(return_value=None)
    config.Config.set = mocker.Mock()
    test_config = config.Config()

    with raises(BadParameter):
        test_config.endpoint = "http://wrongendpoint.com/api/"



from clusterone import client_exceptions, messages

from clusterone.just_client import ClusteroneClient

ClusteroneException = client_exceptions.ClusteroneException
from .auth import authenticate

from clusterone.instances import CLIENT_INSTANCE as client
from clusterone.instances import CONFIG_INSTANCE as config

from .tf_runner import run_tf

#TODO: Solve this better at higher level
# to satisfy clusterone projetcs' dependecy
from clusterone.just_client import get_logs_path, get_data_path

__version__ = '0.13.3a'



import click

from clusterone import authenticate


@click.command()
@click.pass_context
@authenticate()
@click.argument('name')
@click.option('--description', '-d', default='')
@click.option('--repo-path', '-r')
def command(context, name, description, repo_path):
    """
    Create a new project and link existing Git repository in the current directory to Clusterone remote
    """

    # This is required to break circular import and make this Python2.7 compliant
    from clusterone import commands

    cwd = context.obj.cwd

    remote_url = context.invoke(commands.create.project.command, name=name, description=description)

    # since context cannot be referenced in the deault paramters
    repository_path = repo_path if repo_path is not None else cwd

    commands.ln.project.helper.main(context.obj, repository_path, remote_url)

    return



import os
from click.testing import CliRunner

from clusterone import ClusteroneClient
from clusterone.persistance.session import Session
from clusterone.commands.ln.project import cmd
from clusterone.clusterone_cli import cli


def test_remote_from_stdin(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    cmd.main = mocker.Mock()

    CliRunner().invoke(cli, ['ln', 'project'], input="just.another.git.repo.link\n")

    cmd.main.assert_called_with(mocker.ANY, mocker.ANY, "just.another.git.repo.link")

def test_remote_from_project_name(mocker):
    cmd.is_data_on_stdin = mocker.Mock(return_value=False)
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    ClusteroneClient.get_project = mocker.Mock(return_value={'git_auth_link': "my git auth link"})
    cmd.main = mocker.Mock()

    CliRunner().invoke(cli, ['ln', 'project', '--project-path', 'someuser/someProjectName'])

    cmd.main.assert_called_with(mocker.ANY, mocker.ANY, "my git auth link")

def test_no_stdin_no_name_err(mocker):
    cmd.is_data_on_stdin = mocker.Mock(return_value=False)
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    cmd.main = mocker.Mock()

    result = CliRunner().invoke(cli, ['ln', 'project'])

    # This is a fun way of saying internal Clicks MissingParameter Exception
    assert str(result.exception) == '2'

def test_default_repo_path(mocker):
    cmd.is_data_on_stdin = mocker.Mock(return_value=False)
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    ClusteroneClient.get_project = mocker.Mock()
    cmd.main = mocker.Mock()

    CliRunner().invoke(cli, ['ln', 'project', '--project-path', 'someuser/someProjectName'])

    cmd.main.assert_called_with(mocker.ANY, os.getcwd(), mocker.ANY)

def test_user_provided_repo_path(mocker):
    cmd.is_data_on_stdin = mocker.Mock(return_value=False)
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    ClusteroneClient.get_project = mocker.Mock()
    cmd.main = mocker.Mock()

    CliRunner().invoke(
        cli,
        ['ln', 'project', '--project-path', 'someuser/someProjectName', '-r', '/some/path/to/git/repository'], input="")

    cmd.main.assert_called_with(mocker.ANY, '/some/path/to/git/repository', mocker.ANY)

def test_project_aquisition(mocker):
    cmd.is_data_on_stdin = mocker.Mock(return_value=False)
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    ClusteroneClient.get_project = mocker.Mock()
    cmd.main = mocker.Mock()

    CliRunner().invoke(
        cli,
        ['ln', 'project', '--project-path', 'someuser/someProjectName'], input="")

    ClusteroneClient.get_project.assert_called_with("someProjectName", username="someuser")

def test_default_username(mocker):
    cmd.is_data_on_stdin = mocker.Mock(return_value=False)
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    ClusteroneClient.get_project = mocker.Mock()
    cmd.main = mocker.Mock()
    mocker.patch.object(Session, 'get', autospec=True, return_value="defaultusername")

    CliRunner().invoke(
        cli,
        ['ln', 'project', '--project-path', 'someProjectName'], input="")

    ClusteroneClient.get_project.assert_called_with("someProjectName", username="defaultusername")

def test_invalid_project_path(mocker):
    cmd.is_data_on_stdin = mocker.Mock(return_value=False)
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    ClusteroneClient.get_project = mocker.Mock()
    cmd.main = mocker.Mock()

    result = CliRunner().invoke(
        cli,
        ['ln', 'project', '--project-path', 'elorapmordo/////////xd'], input="")

    # This is a fun way of saying internal Clicks Exception
    assert str(result.exception) == '2'



from time import sleep

import click
from click import echo, clear
from clusterone import authenticate
from clusterone.utilities import make_table
from clusterone.client_exceptions import SoftInternalServiceError


HEADER = ['Time', 'Job', 'Job Run ID', 'Status', 'Event']

@click.command()
@click.option(
    '--once',
    is_flag=True,
    help='Display snapshot of latest events.'
    )
@click.pass_obj
@authenticate()
def command(context, once):
    """
    Outputs a contionus stream of events. Press CTRL+C to exit.
    """

    session, client = context.session, context.client

    if once:
        output_events(client)
    else:
        while True:
            output_events(client, before_printing=clear)
            sleep(session.events_refresh_rate)


#TODO: Python2.7 compliance legacy - add * after last positional argument
def output_events(client, before_printing=None):
    """
    Lists events from the system on the stdout
    Before printing action is performed.

    The before_printing action was added as a way to perform
    event aquisition (as it may throw) before clearing the screen
    and therefore not interacting with exception handler
    """

    events_data = extract_data_from_events(client.get_events())

    #TODO: Find the origin of the problem - suspected culprits API returning empty event list - tried to reporoduce - failed. Possible reproduction guidlines: have a valid token, but invalid session.
    try:
        assert events_data
    except AssertionError as exception:
        raise SoftInternalServiceError

    if before_printing:
        before_printing()

    echo(make_table(events_data, header=HEADER))


def extract_data_from_events(events):
    """
    Transform events data into table displayable data
    """

    def get_event_data(event):
        """
        Aquire relevant data from single event
        """

        return [
            event['created_at'][:19],
            event['job_name'],
            event['job_run'],
            event['event_level_display'],
            event['event_type_display'],
        ]

    return [get_event_data(event) for event in events]



from . import create
from . import init
from . import ln
from . import start
from . import stop
from . import get
from . import rm
from . import login
from . import logout
from . import matrix
from . import config
from . import run




from collections import OrderedDict

from click.testing import CliRunner

from clusterone import ClusteroneClient
from clusterone.clusterone_cli import cli
from clusterone.commands.rm.dataset import cmd
from clusterone.client_exceptions import BusyDatasetRemoveAttempt

def test_deletion(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    ClusteroneClient.delete_dataset = mocker.Mock()
    ClusteroneClient.get_jobs = mocker.Mock(return_value=[])
    cmd.path_to_dataset = mocker.Mock(return_value=OrderedDict([
    ('id', 'c385c703-e928-42c6-a47c-383381c280ae'),
    ('name', 'some_dummy_dataset_name'),
    ('owner', OrderedDict([
        ('username', 'someDummyUsername'),
        ])),
    ]))
    mocker.spy(cmd, 'confirm')

    CliRunner().invoke(cli, ['rm', 'dataset', 'someDummyUsername/some_dummy_dataset_name'], 'y')

    cmd.path_to_dataset.assert_called_with(
        'someDummyUsername/some_dummy_dataset_name',
        context=mocker.ANY)

    cmd.confirm.assert_called_with(mocker.ANY, abort=True)

    ClusteroneClient.delete_dataset.assert_called_with(
        'some_dummy_dataset_name',
        'someDummyUsername')

def test_running_jobs(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    ClusteroneClient.delete_dataset = mocker.Mock()
    ClusteroneClient.get_jobs = mocker.Mock(return_value=[OrderedDict([('status', 'started')])])
    cmd.path_to_dataset = mocker.Mock(return_value=OrderedDict([
    ('id', 'c385c703-e928-42c6-a47c-383381c280ae'),
    ('name', 'some_dummy_dataset_name'),
    ('owner', OrderedDict([
        ('username', 'someDummyUsername'),
        ])),
    ]))
    mocker.spy(cmd, 'confirm')

    result = CliRunner().invoke(cli, ['rm', 'dataset', 'dataset'])

    assert isinstance(result.exception, BusyDatasetRemoveAttempt)



from collections import OrderedDict

from click.testing import CliRunner
from pytest import raises

from clusterone import ClusteroneClient
from clusterone.client_exceptions import SoftInternalServiceError
from clusterone.clusterone_cli import cli
from clusterone.commands.get.events import cmd

ORIGINAL_OUTPUT_EVENTS = cmd.output_events
ORIGINAL_EXTRACT_DATA = cmd.extract_data_from_events


def test_one_output(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    cmd.time = mocker.Mock()
    cmd.output_events = mocker.Mock()

    CliRunner().invoke(cli, ['get', 'events', '--once'])

    assert cmd.output_events.call_count == 1


def test_forever_output(mocker):
    cmd.clear = mocker.Mock()
    cmd.output_events = mocker.Mock()

    class RaiseAfterNCalls:
        """
        Raises exception after being called n times

        This is particularly useful when testing infinite loops.
        """

        def __init__(self, exception_class, target_call_count):
            self.call_count = 0
            self.exception_class = exception_class
            self.target_call_count = target_call_count

        def __call__(self, *args, **kwargs):
            self.call_count += 1

            if self.call_count >= self.target_call_count:
                raise self.exception_class("Just testing")

    cmd.sleep = mocker.Mock(side_effect=RaiseAfterNCalls(KeyboardInterrupt, 3))

    CliRunner().invoke(cli, ['get', 'events'])

    # the right thing is called
    cmd.output_events.assert_called_with(mocker.ANY, before_printing=cmd.clear)

    # the right number of times
    assert cmd.sleep.call_count == 3
    assert cmd.output_events.call_count == 3


def test_output_events(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    ClusteroneClient.get_events = mocker.Mock(return_value="retval_from_get_events")
    cmd.extract_data_from_events = mocker.Mock(return_value="retval_from_extract_data")
    cmd.make_table = mocker.Mock(return_value="Example table string")
    cmd.echo = mocker.Mock()

    test_action = mocker.Mock()

    # resetting a mock like a n00b :c
    cmd.output_events = ORIGINAL_OUTPUT_EVENTS

    cmd.output_events(ClusteroneClient(), before_printing=test_action)

    assert ClusteroneClient.get_events.called
    cmd.extract_data_from_events.assert_called_with("retval_from_get_events")
    cmd.make_table.assert_called_with("retval_from_extract_data", header=cmd.HEADER)
    cmd.echo.assert_called_with("Example table string")
    assert test_action.called


def test_extracting_data(mocker):
    cmd.extract_data_from_events = ORIGINAL_EXTRACT_DATA

    assert cmd.extract_data_from_events([
        OrderedDict([
            ('id', 458091),
            ('repository', '0748b269-94f9-48ee-bc84-542c3801acea'),
            ('job', 'd4ad4c61-87b5-4389-855b-02895fa1d369'),
            ('job_run', 'af3da7db-94bf-41b9-acaa-64f175bcae2c'),
            ('job_name', 'muddy-fog-901'),
            ('repository_name', 'clusterone-test-mnist'),
            ('event_level', 40),
            ('event_level_display', 'Error'),
            ('event_type', 'TERMINATE_ERROR'),
            ('event_type_display', 'Terminate the job on error'),
            ('event_content', 'Job terminated due to code error'),
            ('created_at', '2018-03-14T15:36:33.218301Z'),
            ('created_by', None),
        ]),
        OrderedDict([
            ('id', 458090),
            ('repository', '0748b269-94f9-48ee-bc84-542c3801acea'),
            ('job', 'd4ad4c61-87b5-4389-855b-02895fa1d369'),
            ('job_run', 'af3da7db-94bf-41b9-acaa-64f175bcae2c'),
            ('job_name', 'muddy-fog-901'),
            ('repository_name', 'clusterone-test-mnist'),
            ('event_level', 25),
            ('event_level_display', 'Success'),
            ('event_type', 'DELETE_JOB'),
            ('event_type_display', 'Delete Job Command'),
            ('event_content',
             'Job d4ad4c61-87b5-4389-855b-02895fa1d369 Deleted Successfully'
             ),
            ('created_at', '2018-03-14T15:36:32.394262Z'),
            ('created_by', None),
        ]),
    ]) == [
               ['2018-03-14T15:36:33', 'muddy-fog-901', 'af3da7db-94bf-41b9-acaa-64f175bcae2c', 'Error',
                'Terminate the job on error'],
               ['2018-03-14T15:36:32', 'muddy-fog-901', 'af3da7db-94bf-41b9-acaa-64f175bcae2c', 'Success',
                'Delete Job Command'],
           ]


def test_empty_event_list_error(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    ClusteroneClient.get_events = mocker.Mock(return_value=[])

    with raises(SoftInternalServiceError):
        cmd.output_events(ClusteroneClient())



import time

import click

from clusterone import authenticate
from clusterone.messages import dataset_creation_in_progress_message
from clusterone.client_exceptions import RemoteAquisitionFailure
from clusterone import client


def aquire_remote(name, session, client):

    username = session.get('username')
    retry_count, retry_interval = session.retry_count, session.retry_interval

    for _ in range(retry_count):
        time.sleep(retry_interval)

        dataset = client.get_dataset(name, username)
        git_url = dataset['git_auth_link']

        # API response might be evaluated as None or ""
        if not (git_url is None or git_url == ""):
            return git_url

    raise RemoteAquisitionFailure()



@click.command()
@click.pass_obj
@authenticate()
@click.argument('source', default='gitlab')
@click.argument('name', required=False)
@click.option('--description', default='')
def command(context, name, source, description):
    """
    Create a new Clusterone dataset and output its remote URL
    """

    session = context.session

    # handling default case - this should not work like that
    #TODO: Extract polling behavior and dataset handling to sperate classes
    #TODO: Break this into commands
    if not any(map(lambda choice: source in choice, ['s3', 'github', 'gitlab'])):
        name = source
        source = 'gitlab'

    #TODO: Remove this after adding Github support
    if source == 'github':
        raise NotImplemented()

    click.echo(dataset_creation_in_progress_message)

    dataset = client.create_dataset(name, source, description)
    dataset_name = dataset['name']


    if dataset.get('source') == 'gitlab':
        remote_url = dataset.get('git_auth_link')
        # Refactor
        if not remote_url:
            # Try to get remote URL for gitlab
            remote_url = aquire_remote(dataset_name, client=client, session=session)
    else:
        # Handle Github and S3
        remote_url = dataset.get('http_url_to_repo')

    click.echo(remote_url)

    # Warning! This is untested!
    session['current_dataset'], session['current_dataset_name'] = [dataset_name] * 2
    session.save()

    return remote_url



import re as regexp

from setuptools import setup, find_packages


def get_version():
    """
    Aquires version number from insdie the package without utilising Python import

    When installing clusterone from pypi the clusterone python package does not yet
    exist, causing the imports to fail. Yet, this script has access to sourcefiles
    of the project allowing to extract the __version__ info in another way.

    This was tested on Centos, Fedora, Ubuntu Server 14.04 and Windows
    """
    # type: () -> str
    # TODO: Move this to function signature after removing Python 2.7 compliance

    result = regexp.search(r'{}\s*=\s*[\'"]([^\'"]*)[\'"]'.format("__version__"), open("clusterone/__init__.py").read())
    return result.group(1)


def main():
    setup(
        name='clusterone',
        version=get_version(),
        py_modules=[
            'clusterone'
        ],
        packages=find_packages(),
        include_package_data=True,
        # The lowest supported Python 3 version is 3.5
        # please update accordingly iff changes
        python_requires=">2.6, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, <4",
        install_requires=[
            'click',
            'py',
            'coreapi-cli==1.0.9',
            'gitpython',
            'raven',
            'terminaltables',
            'click_log==0.1.8',
            'virtualenv',
            'six',
            # patches Enum functionality for Python 2.7
            'enum34',
        ],
        extras_require={
            'dev': [
                'pytest',
                'pytest-mock',
            ]
        },
        entry_points='''
            [console_scripts]
            just=clusterone.clusterone_cli:main
        ''',

        author="Clusterone",
        author_email="info@clusterone.com",
        description="Clusterone CLI and Python library.",
        license="MIT",
        keywords="",
        url="https://clusterone.com",
    )


if __name__ == "__main__":
    main()



import click
from click.exceptions import BadParameter

from clusterone import authenticate
from .helper import main

@click.command()
@click.option(
    '--dataset-path',
    '-p',
    help="Project identifier. Format \"username/dataset\".",
    required=True
    )
@click.option(
    '--repo-path',
    '-r',
    help="Path to local Git repository.")
@click.pass_obj
@authenticate()
def command(context, dataset_path, repo_path):
    """
    Links existing Clusterone dataset with a local Git repository
    """

    client, session, cwd = context.client, context.session, context.cwd

    # since context cannot be referenced in the deault paramters
    repository_path = repo_path if repo_path is not None else cwd

    path_tokens = dataset_path.split('/')

    if not len(path_tokens) == 2:
        if len(path_tokens) == 1:
            path_tokens = [session.get("username")] + path_tokens
        else:
            raise BadParameter(param_hint="--dataset-path", message="Please provide a valid dataset path. Format: \"username/dataset\".")

    username, dataset_name = path_tokens

    dataset = client.get_dataset(dataset_name, username=username)

    remote_url = dataset.get('git_auth_link')

    main(context, repository_path, remote_url)




import uuid

from clusterone import ClusteroneClient
from clusterone.clusterone_cli import Context

from . import job_serializer, serialize_job

def test_job_id(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    ClusteroneClient.get_job = mocker.Mock(return_value="some test job")
    job_serializer.UUID = mocker.spy(uuid, "UUID")
    context = Context(ClusteroneClient(), None, None)

    result = serialize_job("1c0a06e8-fb8c-454e-a018-29c41ef66cd6", context=context)

    job_serializer.UUID.assert_called_with("1c0a06e8-fb8c-454e-a018-29c41ef66cd6")
    ClusteroneClient.get_job.assert_called_with({"job_id": "1c0a06e8-fb8c-454e-a018-29c41ef66cd6"})
    assert result == "some test job"

def test_job_path(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    ClusteroneClient.get_job = mocker.Mock(return_value="some test job")
    job_serializer.UUID = mocker.Mock(side_effect=ValueError)
    job_serializer.path_to_job_id = mocker.Mock(return_value="some id 12345")
    context = Context(ClusteroneClient(), None, None)

    result = serialize_job("username/project/job", context=context)

    job_serializer.UUID.assert_called_with("username/project/job")
    job_serializer.path_to_job_id.assert_called_with("username/project/job", context=context)
    ClusteroneClient.get_job.assert_called_with({"job_id": "some id 12345"})
    assert result == "some test job"



import click

from clusterone import client
from clusterone.utilities import Choice
from .base_cmd import job_base_options, base


@job_base_options()
@click.option(
    '--instance-type',
    type=Choice(client.instance_types_slugs),
    default="t2.small",
    help="Type of instance")
def command(context, custom_arguments, **kwargs):
    """
    Creates a single-node job.
    """

    job_configuration = base(context, kwargs, module_arguments=custom_arguments)

    job_configuration['parameters']['mode'] = "single"
    job_configuration['parameters']['workers'] = \
    {
        'slug': kwargs['instance_type'],
        'replicas': 1,
    }

    gpu_count = kwargs.get('gpu_count')
    if gpu_count:
        job_configuration['parameters']['workers']['gpu'] = gpu_count

    client.create_job(
        job_configuration['meta']['name'],
        description=job_configuration['meta']['description'],
        parameters=job_configuration['parameters'],
        )



# -*- coding: utf-8 -*-

import click
from click.exceptions import MissingParameter, UsageError

from clusterone import authenticate
from clusterone.utilities import path_to_job_id


@click.command()
@click.pass_obj
@authenticate()
@click.option(
    '--job-id',
    '-i',
    help='ID of the job to start.')
@click.option(
    '--job-path',
    '-p',
    help='Path of the job to start. Format: \"username/project/job-path\".')
def command(context, job_path, job_id):
    """
    Runs an existing job, identified either by name or ID
    """

    client = context.client

    if job_path is None and job_id is None:
        raise MissingParameter(
            "Please provide either of them",
            param_type="--job-id or --job-path")

    if job_path is not None and job_id is not None:
        raise UsageError("Please provide either --job-id or --job-path, but NOT both")

    if job_path:
        job_id = path_to_job_id(job_path, context=context)

    client.start_job(job_id)

    return job_id



import pytest
import git

from clusterone.client_exceptions import LocalRepositoryFailure, LinkAlreadyPresent
from . import helper
from .helper import main


def test_git_repo_check(mocker):
    helper.git.Repo = mocker.Mock()

    main(None, "/some/path/to/repo", "git link")

    helper.git.Repo.assert_called_with("/some/path/to/repo")

def test_git_repo_rethrow(mocker):
    helper.git.Repo = mocker.Mock()
    helper.git.Repo.side_effect = git.InvalidGitRepositoryError()

    with pytest.raises(LocalRepositoryFailure):
        main(None, "/some/path/to/repo", "git link")

def test_link(mocker):
    helper.git.Repo = mocker.Mock()
    instance_mock = mocker.Mock()
    helper.git.Repo.return_value = instance_mock

    main(None, "/some/path/to/repo", "git link")

    instance_mock.create_remote.assert_called_with("clusterone", "git link")

def test_link_failure(mocker):
    helper.git.Repo = mocker.Mock()
    instance_mock = mocker.Mock()
    instance_mock.create_remote.side_effect = git.exc.GitCommandError("command", "status")
    helper.git.Repo.return_value = instance_mock

    with pytest.raises(LinkAlreadyPresent):
        main(None, "/some/path/to/repo", "git link")




# -*- coding: utf-8 -*-

from os import getcwd
from click.testing import CliRunner

from clusterone import ClusteroneClient
from clusterone.clusterone_cli import cli
from clusterone import commands

def test_create_project_all_args(mocker):

    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    commands.create.project.command = mocker.Mock()
    commands.ln.project.helper.main = mocker.Mock()

    CliRunner().invoke(
        cli,
        ['init', 'project', 'more-project-names', '--description', 'This is a sample project description'])

    commands.create.project.command.assert_called_with(name='more-project-names', description='This is a sample project description')


def test__create_project_name_only(mocker):

    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    commands.create.project.command = mocker.Mock()
    commands.ln.project.helper.main = mocker.Mock()

    CliRunner().invoke(cli, ['init', 'project', 'more-project-names'])

    commands.create.project.command.assert_called_with(name='more-project-names', description='')

def test_ln_project_all_params(mocker):

    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    commands.create.project.command = mocker.Mock(return_value="iam.tired.of.inventing.this.stuff.git")
    commands.ln.project.helper.main = mocker.Mock()

    CliRunner().invoke(cli, ['init', 'project', 'more-project-names', '-r', '/sample/repo/path/to/git/repository'])

    commands.ln.project.helper.main.assert_called_with(mocker.ANY, '/sample/repo/path/to/git/repository', "iam.tired.of.inventing.this.stuff.git")

def test_ln_default_repo_path(mocker):

    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    commands.create.project.command = mocker.Mock(return_value="iam.tired.of.inventing.this.stuff.git")
    commands.ln.project.helper.main = mocker.Mock()

    CliRunner().invoke(cli, ['init', 'project', 'more-project-names'])

    commands.ln.project.helper.main.assert_called_with(mocker.ANY, getcwd(), "iam.tired.of.inventing.this.stuff.git")




import click
from click import confirm

from clusterone import authenticate
from clusterone.utilities import serialize_job
from clusterone.client_exceptions import RunningJobRemoveAttempt

@click.command()
@click.pass_obj
@click.argument(
    'job-path-or-id',
    )
@authenticate()
def command(context, job_path_or_id):
    """
    Removes a job, confirmation required
    """

    client = context.client
    job = serialize_job(job_path_or_id, context=context)

    if job['status'] in ['started', 'running']:
        raise RunningJobRemoveAttempt()

    confirm("Removing a job is irreversible and job outputs will be deleted. Are you sure?", abort=True)

    client.delete_job(job['job_id'])



from clusterone import ClusteroneClient

from clusterone.persistance.config import Config
from clusterone.persistance.session import Session


SESSION_INSTANCE=Session()
SESSION_INSTANCE.load()

CONFIG_INSTANCE = Config()

CLIENT_INSTANCE = ClusteroneClient(
    token=SESSION_INSTANCE.get('token'),
    username=SESSION_INSTANCE.get('username'),
    api_url=CONFIG_INSTANCE.endpoint
)




import git

from clusterone.client_exceptions import LocalRepositoryFailure, LinkAlreadyPresent

def main(context, repository_path, remote_url):

    try:
        git_repository = git.Repo(repository_path)
    except git.InvalidGitRepositoryError:
        raise LocalRepositoryFailure()

    try:
        git_repository.create_remote('clusterone', remote_url)
    #TODO: This exception may be too broad
    except git.exc.GitCommandError:
        raise LinkAlreadyPresent()




from click.testing import CliRunner

from clusterone import ClusteroneClient
from clusterone.persistance.session import Session
from clusterone.clusterone_cli import cli

from clusterone.commands.logout import cmd


def test_local_session_deletion(mocker):
    """
    No credentials are left on user's machine
    """

    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    mocker.patch.object(Session, 'delete', autospec=True, return_value=None)

    cmd.is_latest_version = mocker.Mock(return_value=False)

    CliRunner().invoke(cli, ['logout'])

    assert Session.delete.called



import random
import string
import re

import click
from terminaltables import SingleTable, AsciiTable, GithubFlavoredMarkdownTable

MIN_NUM = 100
MAX_NUM = 999
adjs = [
    "autumn", "hidden", "bitter", "misty", "silent", "empty", "dry", "dark",
    "summer", "icy", "delicate", "quiet", "white", "cool", "spring", "winter",
    "patient", "twilight", "dawn", "crimson", "wispy", "weathered", "blue",
    "billowing", "broken", "cold", "damp", "falling", "frosty", "green", "long",
    "late", "lingering", "bold", "little", "morning", "muddy", "old", "red",
    "rough", "still", "small", "sparkling", "throbbing", "shy", "wandering",
    "withered", "wild", "black", "young", "holy", "solitary", "fragrant",
    "aged", "snowy", "proud", "floral", "restless", "divine", "polished",
    "ancient", "purple", "lively", "nameless"
]
nouns = [
    "waterfall", "river", "breeze", "moon", "rain", "wind", "sea", "morning",
    "snow", "lake", "sunset", "pine", "shadow", "leaf", "dawn", "glitter",
    "forest", "hill", "cloud", "meadow", "sun", "glade", "bird", "brook",
    "butterfly", "bush", "dew", "dust", "field", "fire", "flower", "firefly",
    "feather", "grass", "haze", "mountain", "night", "pond", "darkness",
    "snowflake", "silence", "sound", "sky", "shape", "surf", "thunder",
    "violet", "water", "wildflower", "wave", "water", "resonance", "sun",
    "wood", "dream", "cherry", "tree", "fog", "frost", "voice", "paper", "frog",
    "smoke", "star"
]


def random_name_generator():
    return random.choice(adjs) + '-' + random.choice(nouns) + '-' + str(
        random.randint(MIN_NUM, MAX_NUM))


def random_string():
    return ''.join(
        random.SystemRandom().choice(string.ascii_uppercase + string.digits)
        for _ in range(12))


def normalize(lines, string, length):
    """ Normalize a long string into multiple lines.
    A recursive function that will append string to list of lines.
    Args:
        lines: list
        string: string that will be normalized and appended to lines
        length: max length of each line
    """
    if string == '':
        return lines

    words = string.split(' ', 1)
    if len(words) == 2:
        w1, w2 = words[0], words[1]
    elif len(words) < 2:
        w1 = words[0]
        w2 = ''

    if len(lines[-1]) + len(w1) + 1 < length:
        lines[-1] = (lines[-1] + ' ' + w1).strip()
        return normalize(lines, w2, length)
    else:
        if len(w1) < length:
            lines.append(w1)
            return normalize(lines, w2, length)
        else:
            if len(lines[-1]) == 0:
                lines[-1] = w1[:length]
            else:
                lines.append(w1[:length])
            return normalize(lines, (w1[length:] + ' ' + w2).strip(), length)


def normalize_string(string, length):
    """ Normalize a string into multiple lines.
    """
    lines = normalize([''], string, length)
    s = lines[0]
    for line in lines[1:]:
        s += '\n' + line
    return s


def select_valid_index(min_val, max_val, message):
    index = None
    message += ' [%d - %d]' % (min_val, max_val)
    while (index is None) or (not (index >= min_val and index <= max_val)):
        index = click.prompt(text=message, default=min_val, type=int)
    return index


def select_valid_integer(ints, message):
    index = None
    while (index is None) or (not index in ints):
        index = click.prompt(text=message, default=ints[0], type=int)
    return index


def describe(dictionary):
    for key, value in dictionary.items():
        click.echo("  - %s : %s" % (key, value))


def select_repo(repos, name='repository'):
    repo_names = []
    counter = 0
    valid_repos = []
    for repo in repos:
        try:
            click.echo(
                option('%s | %s/%s | Commits: %s' % (counter, repo.get(
                    'owner')['username'], repo.get('name'), len(
                    repo.get('commits', [])))))
            repo_names.append(
                [repo.get('owner')['username'],
                 repo.get('name')])
            counter += 1
            valid_repos.append(repo)
        except:
            pass

    repo_id = select_valid_index(
        0,
        len(valid_repos) - 1,
        question('Enter the id of the desired %s' % name))
    username = valid_repos[repo_id].get('owner')['username']
    repo_name = valid_repos[repo_id].get('name')
    return username, repo_name


def select_job(jobs, message=None):
    click.echo(info('Jobs:'))
    data = []
    data.append(
        ['#', 'Job Name', 'Project', 'Status', 'Launch at'])

    i = 0
    valid_jobs = []
    for job in jobs:
        try:
            data.append([
                i,
                job.get('display_name'),
                '%s/%s:%s' % (job.get('repository_owner'),
                              job.get('repository_name'),
                              job.get('git_commit_hash')[:8]),
                job.get('status'),
                '' if job.get('launched_at') is None else job.get(
                    'launched_at')[:-5]
            ])
            valid_jobs.append(job)
            i += 1
        except:
            pass
    table = render_table(data, max_length=36)
    click.echo(table.table)

    # Select the job
    job_to_pause_id = select_valid_index(
        0, len(valid_jobs) - 1,
        message)
    job_id = valid_jobs[job_to_pause_id].get('job_id')
    job_name = valid_jobs[job_to_pause_id].get('display_name')
    return job_id, job_name


def render_table(data, max_length=24):
    assert (max_length > 0)
    normalized_data = [[normalize_string(str(j), max_length) for j in i]
                       for i in data]
    return GithubFlavoredMarkdownTable(normalized_data)


def tokenize_repo(repo_name):
    try:
        assert (repo_name.count('/') == 1)
        assert (repo_name.count(':') <= 1)
        user, rem = repo_name.split('/')
        rem = rem.split(':')
        repo = rem[0]
        try:
            commit = rem[1]
        except:
            commit = None

        return user, repo, commit
    except:
        return None, None, None


def question(text):
    return " -> %s" % text


def info(text):
    return "%s\n" % text


def option(text):
    return "%s" % text


def special_match(string, search=re.compile(r'[^A-Za-z0-9\-]').search):
    return not bool(search(string))


def repo_name_validator(name, config):
    # Validate name to only have characters and -
    if name is None:
        return False, 'Name can not be None'
    if len(name) == 0:
        return False, 'Name can not be empty'
    if not special_match(name):
        return False, 'Name should only contain characters, numbers and -'
    if not special_match(name[0], search=re.compile(r'[^A-Za-z]').search):
        return False, 'Name should start with characters'
    return True, None


def job_name_validator(name, config):
    pass



#!/usr/bin/env python3

import os
import sys
import logging

import click
import click_log

from clusterone import authenticate

from clusterone.persistance.session import Session
from clusterone import __version__
from clusterone.utils import render_table, info

# TODO: Handle KeyBoardInterrup at high level here

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
HOME_DIR = os.getcwd()
logger = logging.getLogger(__name__)

# Bunch of global messages
session = Session()
session.__init__()
session.load()
if session.get('username') is None:
    owner_help_message = 'Specify owner by usernames'
else:
    owner_help_message = 'Specify owner by username, default: %s' % session.get(
        'username')

pass_config = click.make_pass_decorator(Session, ensure=True)


class Context(object):
    def __init__(self, client, session, cwd):
        self.client = client
        self.session = session
        self.cwd = cwd


from clusterone import ClusteroneException
from clusterone.utilities import log_error


from clusterone import client

@click.group(context_settings=CONTEXT_SETTINGS)
@click.version_option(version=__version__)
@click.pass_context
def cli(context):
    """
    Welcome to the Clusterone Command Line Interface.
    """

    config = Session()
    config.load()
    context.obj = Context(client, config, HOME_DIR)


def main():
    try:
        cli()
    except ClusteroneException as exception:
        log_error(exception)
        sys.exit(exception.exit_code)
    # TODO: THIS
    #except Exception as exception:
    # here Sentry logger is a good idea
    #    pass

# ---------------------------------------

@cli.group()
@click_log.simple_verbosity_option()
@click_log.init(__name__)
@click.pass_obj
def get(context):
    """
    < project(s) | dataset(s) | job(s) | events >
    """
    pass


@cli.group()
@click_log.simple_verbosity_option()
@click_log.init(__name__)
@click.pass_obj
def create(context):
    """
    < project | dataset | job >
    """
    pass


@cli.group()
@click.pass_obj
def rm(context):
    """
    < project | dataset | job >
    """
    pass


@create.group()
@click.pass_obj
def job(context):
    """
    < single | distributed >
    """

    pass


@cli.group()
@click.pass_context
def init(context):
    """
    < project | dataset >
    """
    pass


@cli.group()
@click.pass_context
def ln(config):
    """
    < project | dataset >
    """
    pass


@cli.group()
@click.pass_context
def start(config):
    """
    < job | notebook >
    """
    pass


@cli.group()
@click.pass_context
def stop(config):
    """
    < job | notebook >
    """
    pass


@cli.group()
@click.pass_context
def run(config):
    pass


# TODO: Redo the above to be dynamic -> eg. job command goes through it's modules and lists single | distributed dynamically

# ------------------------

# TODO: Redo this to dynamic import
from clusterone import commands

get.add_command(commands.get.job.command, 'job')
get.add_command(commands.get.events.command, 'events')

create.add_command(commands.create.project.command, 'project')
create.add_command(commands.create.notebook.command, 'notebook')

job.add_command(commands.create.job.single.command, 'single')
job.add_command(commands.create.job.distributed.command, 'distributed')

rm.add_command(commands.rm.job.command, 'job')
rm.add_command(commands.rm.project.command, 'project')
rm.add_command(commands.rm.dataset.command, 'dataset')

get.add_command(commands.get.project.command, 'project')
get.add_command(commands.get.dataset.command, 'dataset')

init.add_command(commands.init.project.command, 'project')

ln.add_command(commands.ln.project.command, 'project')
ln.add_command(commands.ln.dataset.command, 'dataset')

start.add_command(commands.start.job.command, 'job')
start.add_command(commands.start.notebook.command, 'notebook')

create.add_command(commands.create.dataset.command, 'dataset')

stop.add_command(commands.stop.job.command, 'job')
stop.add_command(commands.stop.notebook.command, 'notebook')

cli.add_command(commands.login.command, 'login')
cli.add_command(commands.logout.command, 'logout')

cli.add_command(commands.matrix.command, 'matrix')
cli.add_command(commands.config.command, 'config')

run.add_command(commands.run.local.command, 'local')

# ------------------------

@click.command()
@click.option(
    '--owner', default=session.get('username'), help=owner_help_message)
# @click_log.simple_verbosity_option()
# @click_log.init(__name__)
@click.pass_obj
@authenticate()
def get_jobs(context, owner):
    """
    List jobs
    """
    config = context.session

    # TODO: filter by owner name is not working
    running_jobs = client.get_jobs(params={'owner': owner})
    if running_jobs:
        data = []
        data.append(['#', 'Name', 'Id', 'Project', 'Status', 'Launched at'])

        i = 0
        valid_jobs = []
        for job in running_jobs:
            try:
                data.append([
                    i,
                    '%s/%s' % (job.get('owner'), job.get('display_name')),
                    job['job_id'],
                    '%s/%s:%s' %
                    (job.get('repository_owner'), job.get('repository_name'),
                     job.get('git_commit_hash')[:8]),
                    job.get('status'),
                    '' if job.get('launched_at') is None else job.get(
                        'launched_at')[:-5]
                ])
                i += 1
                valid_jobs.append(job)
            except Exception as exception:
                pass
        table = render_table(data, 36)
        click.echo(table.table)
        return running_jobs
    else:
        click.echo(info("You don't seem to have any jobs yet. Try just create job to make one."))
        return


get.add_command(get_jobs, 'jobs')

@click.command()
@authenticate()
@click.pass_obj
@click.option('--owner')
def get_projects(context, owner=None):
    """
    List projects
    """

    config = context.session

    # TODO: fix owner

    projects = client.get_projects()

    if projects:
        click.echo(info("All projects:"))
        data = []
        data.append(
            ['#', 'Project', 'Created at', 'Description'])

        i = 0
        for project in projects:
            try:
                data.append([
                    i,
                    "%s/%s" % (project.get('owner')
                               ['username'], project.get('name')),
                    project.get('created_at')[:19],
                    project.get('description')
                ])
                i += 1
            except:
                pass
        table = render_table(data, 36)
        click.echo(table.table)
        return projects
    else:
        click.echo(info(
            "No projects found. Use 'just create project' to start a new one."))
        return None


get.add_command(get_projects, 'projects')


@click.command()
@click.option('--owner')
@click_log.simple_verbosity_option()
@click_log.init(__name__)
@click.pass_obj
@authenticate()
def get_datasets(context, owner=None):
    """
    List datasets
    """
    client, config = context.client, context.session

    datasets = client.get_datasets(owner)

    if datasets:
        click.echo(info("All datasets:"))
        data = []
        data.append(
            ['#', 'Dataset', 'Modified at', 'Description'])

        i = 0
        for project in datasets:
            try:
                data.append([
                    i,
                    "%s/%s" % (project.get('owner')
                               ['username'], project.get('name')),
                    project.get('modified_at')[:19],
                    project.get('description')
                ])
                i += 1
            except:
                pass
        table = render_table(data, 36)
        click.echo(table.table)
        return datasets
    else:
        click.echo("It doesn't look like you have any datasets yet. You can create a new one with 'just create dataset'.")
        return None


get.add_command(get_datasets, 'datasets')

#@cli.command()
#@click.argument('path', type=click.Path(exists=True, file_okay=False, resolve_path=True))
#@click.option('--job-id')
#@click_log.simple_verbosity_option()
#@click_log.init(__name__)
#@click.pass_obj
#@authenticate()
#def download_files(context, path, job_id=None):
#    """
#    downloads all outputs and saves at specified path
#    """
#    config, client = context.session, context.client
#    try:
#        # We get list of user jobs and allow user to select them
#        if not job_id:
#            jobs = client.get_jobs()
#            if jobs:
#                job_id, job_name = select_job(jobs, 'Select the job you want to download files from')
#            else:
#                click.echo(
#                    info(
#                        "You do not seem to have any jobs. Use 'tport run' to run a job."
#                    ))
#                return
#        else:
#            job = client.get_job(params={'job_id': job_id})
#            if job:
#                job_id = job.get('job_id')
#                job_name = job.get('display_name')
#            else:
#                click.echo(info("%s is not a valid job id." % job_id))
#                return
#        # Download file list
#        job_file_list = client.get_file_list(job_id)
#        counter = 0
#        for file in job_file_list:
#            counter += 1
#            # Display the files
#            click.echo(
#                option('%s | %s | %s kb' % (counter, file['name'], file['size'] / 1024)))
#            try:
#                if file['size'] > 0:
#                    # Save file in specified path
#                    file_content = client.download_file(job_id, file['name'])
#                    file_path = os.path.join(path, file['name'])
#                    f = open(file_path, "w")
#                    f.write(file_content)
#                    f.close()
#
#            except Exception as e:
#                click.echo("Failed to download files: %s" % str(e))
#                logger.error("Failed to download file: %s" % str(e), exc_info=True)
#                continue
#
#    except Exception as e:
#        logger.error("Failed to download files", exc_info=True)
#        return


if __name__ == '__main__':
    main()



import pytest
from clusterone.client_exceptions import RemoteAquisitionFailure

from clusterone.persistance.session import Session
from clusterone import ClusteroneClient
from clusterone.clusterone_cli import Context

from . import helper
from .helper import main

def test_client_call(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    ClusteroneClient.get_project = mocker.Mock()
    ClusteroneClient.create_project = mocker.Mock()

    helper.time.sleep = mocker.Mock()

    session = Session()
    session.load()
    client = ClusteroneClient(token=session.get('token'), username=session.get('username'))
    context = Context(client, session, None)

    ClusteroneClient.get_project.return_value = {'git_auth_link': "some text"}

    main(context, "ProjectName", "Description...")

    ClusteroneClient.create_project.assert_called_with("ProjectName", "Description...")

def test_remote_aquisition_failure(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    ClusteroneClient.get_project = mocker.Mock()

    helper.time.sleep = mocker.Mock()
    session = Session()
    session.load()
    client = ClusteroneClient(token=session.get('token'), username=session.get('username'))
    context = Context(client, session, None)

    ClusteroneClient.get_project.return_value = {'git_auth_link': None}

    with pytest.raises(RemoteAquisitionFailure):
        main(context, "a1234567890", "sample text sample text")

def test_returns_remote_link(mocker):
    mocker.patch.object(ClusteroneClient, '__init__', autospec=True, return_value=None)
    ClusteroneClient.get_project = mocker.Mock()
    ClusteroneClient.create_project = mocker.Mock()

    helper.time.sleep = mocker.Mock()

    session = Session()
    session.load()
    client = ClusteroneClient(token=session.get('token'), username=session.get('username'))
    context = Context(client, session, None)

    ClusteroneClient.get_project.return_value = {'git_auth_link': "exceptional.remote.link.git"}

    assert main(context, "ProjectName", "Description...") == "exceptional.remote.link.git"




from .main import random_job_name
from .main import silent_prompt
from .log_err import log_error

from .path_to import path_to_project
from .path_to import path_to_job_id
from .path_to import path_to_dataset

from .time_limit_to_minutes import main as time_limit_to_minutes
from .make_table import main as make_table
from .job_serializer import main as serialize_job

from .choice import Choice, lazy_property, lazify

from .version import is_latest_version

from .docstring import append_to_docstring
